import logging
import pytest
import requests
import requests_mock
from datetime import datetime

from waldhart_availability import Duration
from waldhart_availability import WaldhartAvailability

logger = logging.getLogger(__name__)

class TestWaldhartAvailability:
    """
    Test WaldhartAvailability functionality.
    """

    @requests_mock.Mocker(kw="mock")
    def test_login(self, **kwargs):
        kwargs['mock'].post("http://waldhart/login", text="OK")
        kwargs['mock'].get("http://caldav", text="dav")

        waldhart = WaldhartAvailability(waldhart_url="http://waldhart",
                                        waldhart_username="test",
                                        waldhart_password="123",
                                        caldav_url="http://caldav",
                                        caldav_username="testc",
                                        caldav_password="1234")
        r = waldhart.login()
        assert r

    @requests_mock.Mocker(kw="mock")
    def test_read_availability(self, **kwargs):
        kwargs['mock'].post("http://waldhart/login", text="OK")
        kwargs['mock'].get("http://waldhart/myavailability/myavailability_month/get_availability_json?year=2025&month=1", text=JAN_3)
        kwargs['mock'].get("http://caldav", text="dav")

        waldhart = WaldhartAvailability(waldhart_url="http://waldhart",
                                        waldhart_username="test",
                                        waldhart_password="123",
                                        caldav_url="http://caldav",
                                        caldav_username="testc",
                                        caldav_password="1234")
        waldhart.login()

        day = waldhart.read_availability(2025, 1)["3"]
        logger.info(day)

    @requests_mock.Mocker(kw="mock")
    def test_duration(self, **kwargs):
        kwargs['mock'].post("http://waldhart/login", text="OK")
        kwargs['mock'].get("http://caldav", text="dav")

        waldhart = WaldhartAvailability(waldhart_url="http://waldhart",
                                        waldhart_username="test",
                                        waldhart_password="123",
                                        caldav_url="http://caldav",
                                        caldav_username="testc",
                                        caldav_password="1234")
        waldhart.login()

        kwargs['mock'].get("http://waldhart/myavailability/myavailability_month/get_availability_json?year=2025&month=1", text=JAN_3)
        day = waldhart.read_availability(2025, 1)["3"]
        duration = waldhart.duration_type(day)
        assert duration == Duration.ALLDAY

        kwargs['mock'].get("http://waldhart/myavailability/myavailability_month/get_availability_json?year=2025&month=1", text=JAN_6)
        day = waldhart.read_availability(2025, 1)["6"]
        duration = waldhart.duration_type(day)
        assert duration == Duration.AM

        kwargs['mock'].get("http://waldhart/myavailability/myavailability_month/get_availability_json?year=2025&month=1", text=JAN_20)
        day = waldhart.read_availability(2025, 1)["20"]
        duration = waldhart.duration_type(day)
        assert duration == Duration.PM

        kwargs['mock'].get("http://waldhart/myavailability/myavailability_month/get_availability_json?year=2025&month=1", text=JAN_26)
        day = waldhart.read_availability(2025, 1)["26"]
        duration = waldhart.duration_type(day)
        assert duration == Duration.NV

    @requests_mock.Mocker(kw="mock")
    def test_update_availability(self, **kwargs):
        kwargs['mock'].post("http://waldhart/login", text="OK")
        kwargs['mock'].get("mock://caldav", text="dav")

        # TODO: mock PROPFIND requests
        #session = requests.Session()
        #adapter = requests_mock.Adapter()
        #adapter.register_uri("PROPFIND", "mock://caldav", text="dav")
        #session.mount("mock://", adapter)

        waldhart = WaldhartAvailability(waldhart_url="http://waldhart",
                                        waldhart_username="test",
                                        waldhart_password="123",
                                        caldav_url="mock://caldav",
                                        caldav_username="testc",
                                        caldav_password="1234")
        waldhart.login()

        # expect ALLDAY
        kwargs['mock'].get("http://waldhart/myavailability/myavailability_month/get_availability_json?year=2025&month=1", text=JAN_3)
        day = waldhart.read_availability(2025, 1)["3"]
        duration = waldhart.duration_type(day)
        waldhart.update_availability(2025, 1, 3, duration)

        # expect forenoon (AM)
        kwargs['mock'].get("http://waldhart/myavailability/myavailability_month/get_availability_json?year=2025&month=1", text=JAN_6)
        day = waldhart.read_availability(2025, 1)["6"]
        duration = waldhart.duration_type(day)
        waldhart.update_availability(2025, 1, 6, duration)

        # expect afternoon (PM)
        kwargs['mock'].get("http://waldhart/myavailability/myavailability_month/get_availability_json?year=2025&month=1", text=JAN_20)
        day = waldhart.read_availability(2025, 1)["20"]
        duration = waldhart.duration_type(day)
        waldhart.update_availability(2025, 1, 20, duration)

        # expect not available (NV)
        kwargs['mock'].get("http://waldhart/myavailability/myavailability_month/get_availability_json?year=2025&month=1", text=JAN_26)
        day = waldhart.read_availability(2025, 1)["26"]
        duration = waldhart.duration_type(day)
        waldhart.update_availability(2025, 1, 26, duration)


# Waldhart json response mock

JAN_3 = """
{
  "3": {
    "has_course_pm": false,
    "has_course_md": false,
    "pm": true,
    "on_call": false,
    "ready_3": null,
    "ready_1": null,
    "to_2": null,
    "has_course_am": false,
    "md": true,
    "group_type": null,
    "to_1": null,
    "vacation": false,
    "from_2": null,
    "has_course_all_day": false,
    "am": true,
    "ready_2": null,
    "from_3": null,
    "ready_state": 1,
    "from_1": null,
    "ill": false,
    "courses": null,
    "show_hatching": true,
    "not_available": false,
    "group_type_2": null,
    "to_3": null,
    "has_course": false
  }
}
"""

JAN_6 = """
{
  "6": {
    "has_course_pm": false,
    "has_course_md": false,
    "pm": false,
    "on_call": false,
    "ready_3": 0,
    "ready_1": 1,
    "to_2": 840,
    "has_course_am": false,
    "md": false,
    "group_type": null,
    "to_1": 720,
    "vacation": false,
    "from_2": 720,
    "has_course_all_day": false,
    "am": true,
    "ready_2": 0,
    "from_3": 720,
    "ready_state": 5,
    "from_1": 480,
    "ill": false,
    "courses": null,
    "show_hatching": true,
    "not_available": false,
    "group_type_2": null,
    "to_3": 960,
    "has_course": false
  }
}
"""

JAN_20 = """
{
  "20": {
    "has_course_pm": false,
    "has_course_md": false,
    "pm": true,
    "on_call": false,
    "ready_3": 1,
    "ready_1": 0,
    "to_2": 840,
    "has_course_am": false,
    "md": false,
    "group_type": null,
    "to_1": 720,
    "vacation": false,
    "from_2": 720,
    "has_course_all_day": false,
    "am": false,
    "ready_2": 0,
    "from_3": 720,
    "ready_state": 5,
    "from_1": 480,
    "ill": false,
    "courses": null,
    "show_hatching": true,
    "not_available": false,
    "group_type_2": null,
    "to_3": 960,
    "has_course": false
  }
}
"""

JAN_26 = """
{
  "26": {
    "has_course_pm": false,
    "has_course_md": false,
    "pm": false,
    "on_call": false,
    "ready_3": null,
    "ready_1": null,
    "to_2": null,
    "has_course_am": false,
    "md": false,
    "group_type": null,
    "to_1": null,
    "vacation": false,
    "from_2": null,
    "has_course_all_day": false,
    "am": false,
    "ready_2": null,
    "from_3": null,
    "ready_state": 0,
    "from_1": null,
    "ill": false,
    "courses": null,
    "show_hatching": true,
    "not_available": true,
    "group_type_2": null,
    "to_3": null,
    "has_course": false
  }
}
"""
